import errno
import os
import re
import uuid

from severino.sdk import MailCarrier, MailCarrierTemplate


# Helper class to work with templates classes in code
class MailTemplate:
    _template_name: str
    _html_path: os.path
    _connection: uuid

    def __init__(self) -> None:
        self.mail_carrier = MailCarrier()
        self.mail_carrier_template = MailCarrierTemplate()

    @property
    def connection(self) -> uuid:
        return self._connection

    @connection.setter
    def connection(self, value: uuid):
        self._connection = value

    @property
    def html_path(self) -> os.path:
        return self._html_path

    @html_path.setter
    def html_path(self, value: os.path):
        if not os.path.exists(value):
            raise FileNotFoundError(errno.ENOENT, os.strerror(errno.ENOENT), value)

        self._html_path = value

    @property
    def template_name(self) -> str:
        return self._template_name

    @template_name.setter
    def template_name(self, value: str):
        if not re.match(r"^[a-z_-]+$", value):
            raise Exception(f"'{value}' is not a valid name for template.")

        self._template_name = value

    def mail_migrate(self) -> None:
        """Update or Create email in Severino server if not exist"""
        self.mail_carrier_template.update(
            template_name=self.template_name,
            template_path=self.html_path,
        )

    def delete_remote(self) -> None:
        """Delete self template in Severino server"""
        self.mail_carrier_template.delete(template_name=self.template_name)

    def mail_preview(self, context_vars: dict = None) -> None:
        """Make preview email in your localhost
        Args:
            context_vars (dict, optional): dictionary with template variables
        """
        self.mail_carrier_template.test_template(
            template_path=self.html_path, context_vars=context_vars
        )

    def send_email(
        self,
        subject: str,
        to_email: list[str],
        bcc: list[str] = None,
        cc: list[str] = None,
        reply_to: list = None,
        context_vars: dict = None,
        files: list = [],
        tag: str = None,
    ):
        """Send email using this template.
        Args:
            subject (str, mandatory): The subject line of the email.
            to_email (list): A list of recipient addresses. Recipients are the audience of the message.
            bcc (list, optional): A list of addresses used in the “Bcc” header when sending the email. Recipients are those being discreetly or surreptitiously informed of the communication and cannot be seen by any of the other addressees. P.s: It is common practice to use the BCC: field when addressing a very long list of recipients, or a list of recipients that should not (necessarily) know each other, e.g. in mailing lists.
            cc (list, optional): A list of recipient addresses. Recipients are others whom the author wishes to publicly inform of the message (carbon copy)
            reply_to (list, optional): A list of recipient addresses used in the “Reply-To” header when sending the email.
            context_vars (dict, optional): An object containing key and values ​​that will be used by the template.
            files (list, optional): A list containing the files that will be sent together to the email. E.g: [{"name": "file.pdf", "file": "PCFkb2N0eXBlIGh0bWw+..."}]
            tag (str, optional): You can add a tag to find this email in the future.
        """
        print("test")
        return self.mail_carrier.send(
            from_connection=self._connection,
            template_name=self.template_name,
            to_email=to_email,
            bcc=bcc,
            cc=cc,
            reply_to=reply_to,
            subject=subject,
            context_vars=context_vars,
            files=files,
            tag=tag,
        )



from severino.sdk.airflow import Airflow
from severino.sdk.hub_distance import HubDistance, HubDistanceCEP
from severino.sdk.internal_recruitment import MindsightReminder, Movement, Stage
from severino.sdk.mail_carrier import MailCarrier, MailCarrierTemplate
from severino.sdk.reports_of_pcds_in_admission import ReminderReportsOfPcdsInAdmission
from severino.sdk.rescission_workflow import RescissionWorkflow
from severino.sdk.store_data import StoreData, StoreDataTmp
from severino.sdk.vagas_dot_com import (
    VagasDotComBot,
    VagasDotComCandidateGHLoad,
    VagasDotComCandidateMigration,
    VagasDotComEmailGHMigration,
)



import base64
import http.server
import random
import socketserver
import uuid

from jinja2 import BaseLoader, Environment

from severino.sdk.helpers.http_requests import Http
from severino.settings import SEVERINO_API_URL


class MailCarrier:
    def __init__(self):
        self.http = Http()
        self.severino_api_url = SEVERINO_API_URL
        self.path = "/mail-carrier"

    def last_email_sent_to(self, email: str):
        return self.http.get(
            url=f"{self.severino_api_url}{self.path}/last-email-sent-to/{email}/"
        )

    def list(self, filters: dict = {}):
        """List

        Args:
            filters (dict, optional): List of filters: to_mail, tag. E.g: {"tag": "xyz"}
        """
        return self.http.get(
            url=f"{self.severino_api_url}{self.path}/mail/", params=filters
        )

    def send(
        self,
        from_connection: uuid,
        template_name: str,
        to_email: list,
        bcc: list = None,
        cc: list = None,
        reply_to: list = None,
        subject: str = "",
        context_vars: dict = {},
        files: list = [],
        tag: str = None,
    ):
        """Send emails using Severino.

        Args:
            from_connection (uuid): UUID that identifies the connection (credentials) that will be used to send the email.
            template_name (str): Name of the template to be used for sending the email.
            to_email (list): A list of recipient addresses. Recipients are the audience of the message.
            bcc (list, optional): A list of addresses used in the “Bcc” header when sending the email. Recipients are those being discreetly or surreptitiously informed of the communication and cannot be seen by any of the other addressees. P.s: It is common practice to use the BCC: field when addressing a very long list of recipients, or a list of recipients that should not (necessarily) know each other, e.g. in mailing lists.
            cc (list, optional): A list of recipient addresses. Recipients are others whom the author wishes to publicly inform of the message (carbon copy)
            reply_to (list, optional): A list of recipient addresses used in the “Reply-To” header when sending the email.
            subject (str, optional): The subject line of the email.
            context_vars (dict, optional): An object containing key and values ​​that will be used by the template.
            files (list, optional): A list containing the files that will be sent together to the email. E.g: [{"name": "file.pdf", "file": "PCFkb2N0eXBlIGh0bWw+..."}]
            tag (str, optional): You can add a tag to find this email in the future.
        """

        data = {
            "subject": subject,
            "to_email": to_email,
            "bcc": bcc,
            "cc": cc,
            "reply_to": reply_to,
            "from_connection": from_connection,
            "template_name": template_name,
            "context_vars": context_vars,
            "tag": tag,
        }

        if files:
            data["base64_files"] = []

            for file in files:
                data["base64_files"].append(
                    {
                        "name": file["name"],
                        "content": self.__get_file(file=file["file"]),
                    }
                )

        return self.http.post(
            url=f"{self.severino_api_url}{self.path}/send/", data=data
        )

    def __get_file(self, file):
        if isinstance(file, str):
            file = open(file, "rb")
            file = file.read()

        return base64.b64encode(file).decode("utf8")


class MailCarrierTemplate:
    def __init__(self, port: int = None):
        self.http = Http()
        self.severino_api_url = SEVERINO_API_URL
        self.path = "/mail-carrier/mail-template"
        self.port = port
        if not port:
            self.port = random.randint(8400, 8700)

    def test_template(self, template_path: str, context_vars: dict):
        class HttpRequestHandler(http.server.SimpleHTTPRequestHandler):
            def do_GET(self):
                if self.path == "/mail/template":
                    html = ""

                    # Sending an '200 OK' response
                    self.send_response(200)

                    # Setting the header
                    self.send_header("Content-type", "text/html")

                    # Whenever using 'send_header', you also have to call 'end_headers'
                    self.end_headers()

                    with open(template_path, "rb") as file:
                        html = file.read()

                    template = Environment(loader=BaseLoader()).from_string(
                        html.decode("utf-8")
                    )

                    # Writing the HTML contents with UTF-8
                    self.wfile.write(
                        bytes(str(template.render(**context_vars)), "utf8")
                    )

                return

        # Create an object of the above class
        handler_object = HttpRequestHandler

        PORT = self.port

        severino_server = socketserver.TCPServer(("", PORT), handler_object)

        print("\n")

        print(
            "Severino SDK server started at http://localhost:"
            + str(PORT)
            + "/mail/template"
        )

        try:
            severino_server.serve_forever()
            severino_server.shutdown()
        except:
            severino_server.shutdown()

    def update(self, template_name: str, template_path: str):
        template_data = None
        new = False

        check_if_exists = self.http.get(
            url=f"{self.severino_api_url}{self.path}/", params={"name": template_name}
        ).json()

        if not check_if_exists["count"] > 0:
            template_data = self.http.post(
                url=f"{self.severino_api_url}{self.path}/",
                data={
                    "name": template_name,
                    "base64_html": self.__get_file(template_path),
                },
            ).json()
            new = True

        if not template_data:
            template_data = self.http.put(
                url=f"{self.severino_api_url}{self.path}/{check_if_exists['results'][0]['id']}/",
                data={
                    "name": template_name,
                    "base64_html": self.__get_file(template_path),
                },
            ).json()

        if new:
            print("\x1b[6;30;42m" + " Template created successfully " + "\x1b[0m")
            return

        print("\x1b[6;30;42m" + " Template updated successfully " + "\x1b[0m")
        return

    def delete(self, template_name):
        check_if_exists = self.http.get(
            url=f"{self.severino_api_url}{self.path}/", params={"name": template_name}
        ).json()

        if not check_if_exists["count"] > 0:
            print("\033[31m" + " Template not found ")
            return

        self.http.delete(
            url=f"{self.severino_api_url}{self.path}/{check_if_exists['results'][0]['id']}/"
        )

        print("\x1b[6;30;42m" + " Template deleted successfully " + "\x1b[0m")

    def __get_file(self, file_path):
        file = open(file_path, "rb")
        file = file.read()
        return base64.b64encode(file).decode("utf8")



from severino.sdk.helpers.http_requests import Http


class PaginationControls:
    LAST_REQUEST = None

    def __init__(self, request) -> None:
        self.LAST_REQUEST = request

    def __control(self, action):
        data = None
        response = None

        if self.LAST_REQUEST:
            data = self.LAST_REQUEST.json()

        if data:
            if data["links"][action]:
                response = Http()._http_request(
                    method="GET",
                    url=data["links"][action],
                    headers=dict(self.LAST_REQUEST.request.headers),
                )
                self.LAST_REQUEST = response
        return self

    @property
    def page_size(self):
        data = self.LAST_REQUEST.json()
        return data["page_size"]

    @property
    def total_pages(self):
        data = self.LAST_REQUEST.json()
        return data["total_pages"]

    @property
    def current_page(self):
        data = self.LAST_REQUEST.json()
        return data["page"]

    @property
    def count(self):
        data = self.LAST_REQUEST.json()
        return data["count"]

    @property
    def is_first_page(self):
        data = self.LAST_REQUEST.json()
        return data["page"] == 1

    @property
    def is_last_page(self):
        data = self.LAST_REQUEST.json()
        return data["page"] == data["total_pages"]

    @property
    def raw_data(self):
        return self.LAST_REQUEST.json()

    @property
    def data(self):
        return self.LAST_REQUEST.json()["results"]

    def request(self):
        return self.LAST_REQUEST

    def next(self):
        return self.__control("next")

    def previous(self):
        return self.__control("previous")



from severino.sdk.helpers.http_requests import Http
from severino.settings import SEVERINO_API_URL


class Airflow:
    def __init__(self):
        self.http = Http()
        self.severino_api_url = SEVERINO_API_URL
        self.path = "/airflow/dag"

    def create(self, dag_id: str):
        return self.http.post(
            url=f"{self.severino_api_url}{self.path}/", data={"dag_id": dag_id}
        )

    def read(self, dag_id: str):
        return self.http.get(url=f"{self.severino_api_url}{self.path}/{dag_id}/")

    def list(self):
        return self.http.get(url=f"{self.severino_api_url}{self.path}/")

    def update(self, airflow_id: str, dag_id: str):
        return self.http.put(
            url=f"{self.severino_api_url}{self.path}/{airflow_id}/",
            data={"dag_id": dag_id},
        )

    def delete(self, airflow_id):
        return self.http.delete(url=f"{self.severino_api_url}{self.path}/{airflow_id}")



import datetime

from severino.sdk.helpers.http_requests import Http
from severino.settings import SEVERINO_API_URL


class GHMigration:
    def __init__(self):
        self.http = Http()
        self.severino_api_url = SEVERINO_API_URL
        self.path = "/vagas-dot-com/gh/"

    def create(
        self,
        last_migration_at: datetime,
    ):
        return self.http.post(
            url=f"{self.severino_api_url}{self.path}/",
            data={
                "last_migration_at": last_migration_at.strftime("%Y-%m-%d %H:%M:%S"),
            },
        )

    def read(self, migration_uuid: str):
        """Read

        Args:
            migration_uuid (str): Migration id

        Returns:
            request: Request object
        """
        return self.http.get(
            url=f"{self.severino_api_url}{self.path}/{migration_uuid}/"
        )

    def last_migration_at(self):
        """Last migration at

        Returns:
            request: Request object
        """
        return self.http.get(
            url=f"{self.severino_api_url}{self.path}/last-migration-at/"
        )

    def list(self, filters: dict = {}):
        """List

        Args:
            filters (dict, optional): List of filters: vagas_job_id, gh_job_id, last_migration_at E.g: {"vagas_job_id": 999999}.

        Returns:
            _type_: _description_
        """
        return self.http.get(url=f"{self.severino_api_url}{self.path}/", params=filters)

    def update(
        self,
        migration_uuid: str,
        last_migration_at: datetime = None,
    ):
        data = {"last_migration_at": None}

        if last_migration_at:
            data["last_migration_at"] = last_migration_at.strftime("%Y-%m-%d %H:%M:%S")

        return self.http.put(
            url=f"{self.severino_api_url}{self.path}/{migration_uuid}/",
            data=data,
        )

    def delete(self, migration_uuid):
        return self.http.delete(
            url=f"{self.severino_api_url}{self.path}/{migration_uuid}/"
        )



from uuid import UUID

from severino.sdk.helpers.http_requests import Http
from severino.settings import SEVERINO_API_URL


class HubDistanceCEP:
    http = None
    severino_api_url = ""
    path = ""

    def __init__(self):
        self.http = Http()
        self.severino_api_url = SEVERINO_API_URL
        self.path = "/hub-distance/cep"

    def get(self, cep: str):
        """Get CEP Info

        Args:
            cep (str)
        """
        return self.http.get(url=f"{self.severino_api_url}{self.path}/{cep}/")

    def create(
        self,
        latitude: str,
        longitude: str,
        cep: str,
        address: str = "",
        neighborhood: str = "",
        city: str = "",
        state: str = "",
    ):
        """Add new CEP info

        Args:
            latitude (str)
            longitude (str)
            cep (str)
            address (str, optional)
            neighborhood (str, optional)
            city (str, optional)
            state (str, optional)
        """

        if not "-" in cep:
            cep = cep[:5] + "-" + cep[-3:]

        data = {
            "latitude": latitude,
            "longitude": longitude,
            "cep": cep,
            "address": address,
            "neighborhood": neighborhood,
            "city": city,
            "state": state,
        }

        return self.http.post(
            url=f"{self.severino_api_url}{self.path}/",
            data=data,
        )

    def put(
        self,
        cep_id: UUID,
        latitude: str,
        longitude: str,
        cep: str,
        address: str = "",
        neighborhood: str = "",
        city: str = "",
        state: str = "",
    ):
        """Update CEP info

        Args:
            latitude (str)
            longitude (str)
            cep (str)
            address (str, optional)
            neighborhood (str, optional)
            city (str, optional)
            state (str, optional)
        """

        if not "-" in cep:
            cep = cep[:5] + "-" + cep[-3:]

        data = {
            "latitude": latitude,
            "longitude": longitude,
            "cep": cep,
            "address": address,
            "neighborhood": neighborhood,
            "city": city,
            "state": state,
        }

        return self.http.put(
            url=f"{self.severino_api_url}{self.path}/{cep_id}/",
            data=data,
        )


class HubDistance:
    def __init__(self):
        self.http = Http()
        self.severino_api_url = SEVERINO_API_URL
        self.path = "/hub-distance"

    def distance(self, cep: str):
        """Get the distance between CEP and Hub

        Args:
            cep (str): E.g: 06401-000

        Returns:
            object: Http requests object
        """
        return self.http.get(url=f"{self.severino_api_url}{self.path}/{cep}/")



from datetime import datetime

from severino.sdk.helpers.http_requests import Http
from severino.sdk.helpers.pagination import PaginationControls
from severino.settings import SEVERINO_API_URL


class ReminderReportsOfPcdsInAdmission:
    def __init__(self):
        self.http = Http()
        self.severino_api_url = SEVERINO_API_URL
        self.path = "/reports-of-pcds-in-admission/reminder"

    def create(
        self,
        candidate_id: str = "",
        candidate_name: str = "",
        candidate_cpf: str = "",
        status_code: str = "",
        status_name: str = "",
    ):
        return self.http.post(
            url=f"{self.severino_api_url}{self.path}/",
            data={
                "candidate_id": candidate_id,
                "candidate_name": candidate_name,
                "candidate_cpf": candidate_cpf,
                "status_code": status_code,
                "status_name": status_name,
            },
        )

    def read(self, movement_uuid: str):
        return self.http.get(url=f"{self.severino_api_url}{self.path}/{movement_uuid}/")

    def list(self, filters: dict = {}):
        """List

        Args:
            filters (dict, optional): List of filters: candidate_id, candidate_cpf, status_code, status_name. E.g: {"candidate_cpf": "99999999999"}.

        Returns:
            _type_: _description_
        """
        response = self.http.get(
            url=f"{self.severino_api_url}{self.path}/", params=filters
        )
        return PaginationControls(response)

    def update(
        self,
        movement_uuid: str,
        last_mail_sent_at: datetime = None,
        reminders_sent: int = 0,
        candidate_id: str = "",
        candidate_name: str = "",
        candidate_cpf: str = "",
        status_code: str = "",
        status_name: str = "",
    ):
        data = {
            "candidate_id": candidate_id,
            "reminders_sent": reminders_sent,
            "candidate_name": candidate_name,
            "candidate_cpf": candidate_cpf,
            "status_code": status_code,
            "status_name": status_name,
        }

        if last_mail_sent_at:
            data["last_mail_sent_at"] = last_mail_sent_at.strftime("%Y-%m-%d %H:%M:%S")

        return self.http.put(
            url=f"{self.severino_api_url}{self.path}/{movement_uuid}/",
            data=data,
        )

    def delete(self, movement_uuid):
        return self.http.delete(
            url=f"{self.severino_api_url}{self.path}/{movement_uuid}/"
        )



from datetime import datetime
from uuid import UUID

from severino.sdk.helpers.http_requests import Http
from severino.settings import SEVERINO_API_URL


class VagasDotComEmailGHMigration:
    def __init__(self):
        self.http = Http()
        self.severino_api_url = SEVERINO_API_URL
        self.path = "/vagas-dot-com/gh"

    def create(self, last_migration_at: datetime):
        return self.http.post(
            url=f"{self.severino_api_url}{self.path}/migration/",
            data={
                "last_migration_at": last_migration_at.strftime("%Y-%m-%d %H:%M:%S"),
            },
        )

    def read(self, migration_uuid: str):
        return self.http.get(
            url=f"{self.severino_api_url}{self.path}/migration/{migration_uuid}/"
        )

    def list(self, filters: dict = {}):
        """List

        Args:
            filters (dict, optional): List of filters: last_migration_gte and last_migration_lte E.g: {"last_migration_gte": "2023-05-31 00:00:00", "last_migration_lte": "2023-05-31 23:59:59"}.

        Returns:
            _type_: _description_
        """
        return self.http.get(
            url=f"{self.severino_api_url}{self.path}/migration/", params=filters
        )

    def last_migration(self):
        """List

        Args:
            filters (dict, optional): List of filters: last_migration_gte and last_migration_lte E.g: {"last_migration_gte": "2023-05-31 00:00:00", "last_migration_lte": "2023-05-31 23:59:59"}.

        Returns:
            _type_: _description_
        """
        return self.http.get(
            url=f"{self.severino_api_url}{self.path}/migration/last-migration-at"
        )

    def update(self, migration_uuid: UUID, last_migration_at: datetime):
        return self.http.put(
            url=f"{self.severino_api_url}{self.path}/migration/{migration_uuid}/",
            data={
                "last_migration_at": last_migration_at.strftime("%Y-%m-%d %H:%M:%S"),
            },
        )

    def delete(self, migration_uuid):
        return self.http.delete(
            url=f"{self.severino_api_url}{self.path}/migration/{migration_uuid}/"
        )


class VagasDotComBot:
    def __init__(self):
        self.http = Http()
        self.severino_api_url = SEVERINO_API_URL
        self.path = "/vagas-dot-com/bot"

    def create(
        self,
        vagas_job_id: str,
        gh_job_id: str,
        last_migration_at: datetime,
    ):
        return self.http.post(
            url=f"{self.severino_api_url}{self.path}/",
            data={
                "vagas_job_id": vagas_job_id,
                "gh_job_id": gh_job_id,
                "last_migration_at": last_migration_at.strftime("%Y-%m-%d %H:%M:%S"),
            },
        )

    def read(self, migration_uuid: str):
        return self.http.get(
            url=f"{self.severino_api_url}{self.path}/{migration_uuid}/"
        )

    def list(self, filters: dict = {}):
        """List

        Args:
            filters (dict, optional): List of filters: vagas_job_id, gh_job_id, last_migration_at E.g: {"vagas_job_id": 999999}.

        Returns:
            _type_: _description_
        """
        return self.http.get(url=f"{self.severino_api_url}{self.path}/", params=filters)

    def update(
        self,
        migration_uuid: str,
        vagas_job_id: str = None,
        gh_job_id: str = None,
        last_migration_at: datetime = None,
    ):
        data = {"vagas_job_id": vagas_job_id, "gh_job_id": gh_job_id}

        if last_migration_at:
            data["last_migration_at"] = last_migration_at.strftime("%Y-%m-%d %H:%M:%S")

        data = {key: data[key] for key in data if not key}

        return self.http.put(
            url=f"{self.severino_api_url}{self.path}/{migration_uuid}/",
            data=data,
        )

    def delete(self, migration_uuid):
        return self.http.delete(
            url=f"{self.severino_api_url}{self.path}/{migration_uuid}/"
        )


class VagasDotComCandidateMigration:
    def __init__(self):
        self.http = Http()
        self.severino_api_url = SEVERINO_API_URL
        self.path = "/vagas-dot-com/candidate-migration"

    def create(
        self,
        migration_uuid: UUID,
        name: str,
        city: str = "",
        most_recent_professional_experience: str = "",
        vagas_candidate_id: str = "",
    ):
        return self.http.post(
            url=f"{self.severino_api_url}{self.path}/",
            data={
                "name": name,
                "city": city,
                "most_recent_professional_experience": most_recent_professional_experience,
                "vagas_candidate_id": vagas_candidate_id,
                "migration": migration_uuid,
            },
        )

    def read(self, candidate_migration_uuid: str):
        return self.http.get(
            url=f"{self.severino_api_url}{self.path}/{candidate_migration_uuid}/"
        )

    def list(self, filters: dict = {}):
        """List

        Args:
            filters (dict, optional): List of filters: vagas_candidate_id E.g: {"vagas_candidate_id": "999999"}.

        Returns:
            _type_: _description_
        """
        return self.http.get(url=f"{self.severino_api_url}{self.path}/", params=filters)

    def update(
        self,
        candidate_migration_uuid: UUID,
        migration_uuid: UUID,
        name: str,
        city: str,
        most_recent_professional_experience: str,
        vagas_candidate_id: str,
    ):
        return self.http.put(
            url=f"{self.severino_api_url}{self.path}/{candidate_migration_uuid}/",
            data={
                "name": name,
                "city": city,
                "most_recent_professional_experience": most_recent_professional_experience,
                "vagas_candidate_id": vagas_candidate_id,
                "migration": migration_uuid,
            },
        )

    def delete(self, candidate_migration_uuid):
        return self.http.delete(
            url=f"{self.severino_api_url}{self.path}/{candidate_migration_uuid}/"
        )


class VagasDotComCandidateGHLoad:
    def __init__(self):
        self.http = Http()
        self.severino_api_url = SEVERINO_API_URL
        self.path = "/vagas-dot-com/candidate-gh-load"

    def create(
        self,
        name: str,
        email: str,
        gh_candidate_id: str,
        status: str,
        vagas_candidate_id: str = None,
    ):
        candidate_migration = {}
        if vagas_candidate_id:
            candidate_migration = (
                VagasDotComCandidateMigration()
                .list(filters={"vagas_candidate_id": vagas_candidate_id})
                .json()
            )

        candidate_migration_uuid = None
        if candidate_migration and candidate_migration.get("count", 0) > 0:
            candidate_migration_uuid = candidate_migration["results"][0]["id"]

        return self.http.post(
            url=f"{self.severino_api_url}{self.path}/",
            data={
                "candidate_migration": candidate_migration_uuid,
                "name": name,
                "email": email,
                "gh_candidate_id": gh_candidate_id,
                "status": status,
            },
        )

    def read(self, candidate_gh_load_uuid: str):
        return self.http.get(
            url=f"{self.severino_api_url}{self.path}/{candidate_gh_load_uuid}/"
        )

    def list(self, filters: dict = {}):
        """List

        Args:
            filters (dict, optional): List of filters: gh_candidate_id E.g: {"gh_candidate_id": "999999"}.

        Returns:
            _type_: _description_
        """
        return self.http.get(url=f"{self.severino_api_url}{self.path}/", params=filters)

    def update(
        self,
        candidate_gh_load_uuid: UUID,
        candidate_migration_uuid: UUID,
        name: str,
        email: str,
        gh_candidate_id: str,
        status: str,
    ):
        return self.http.put(
            url=f"{self.severino_api_url}{self.path}/{candidate_gh_load_uuid}/",
            data={
                "candidate_migration": candidate_migration_uuid,
                "name": name,
                "email": email,
                "gh_candidate_id": gh_candidate_id,
                "status": status,
            },
        )

    def delete(self, candidate_gh_load_uuid):
        return self.http.delete(
            url=f"{self.severino_api_url}{self.path}/{candidate_gh_load_uuid}/"
        )



from datetime import datetime

from severino.sdk.helpers.http_requests import Http
from severino.settings import SEVERINO_API_URL


class RescissionWorkflow:
    def __init__(self):
        self.http = Http()
        self.severino_api_url = SEVERINO_API_URL
        self.path = "/rescission-workflow"

    def create(
        self,
        process_code: int = None,
        process_status: int = None,
        start_date: datetime = None,
        completion_date: datetime = None,
        applicant_name: str = None,
        target_company_code: int = None,
        target_enrollment: int = None,
        target_name: str = None,
        effective_position_description: str = None,
        rescission_date: datetime = None,
        notice_date: datetime = None,
        target_admission_date: datetime = None,
        code_reason_rescission: int = None,
        activity_name: str = None,
        rescission_time: str = None,
        face_to_face_communication: str = None,
        email_corp_witness_1: str = None,
        email_corp_witness_2: str = None,
        workplace: str = None,
        zip_code: str = None,
        complete_address: str = None,
        private_email: str = None,
        private_phone: str = None,
        corporate_cell_phone: str = None,
        corporate_email: str = None,
        return_computer: bool = None,
        return_chair: bool = None,
        return_other_items: bool = None,
        access_data_center: bool = None,
        servicenow_sys_id: str = None,
        servicenow_ticket_number: str = None,
    ):
        return self.http.post(
            url=f"{self.severino_api_url}{self.path}/",
            data={
                "process_code": process_code,
                "process_status": process_status,
                "start_date": start_date.strftime("%Y-%m-%d") if start_date else None,
                "completion_date": completion_date.strftime("%Y-%m-%d")
                if completion_date
                else None,
                "applicant_name": applicant_name,
                "target_company_code": target_company_code,
                "target_enrollment": target_enrollment,
                "target_name": target_name,
                "effective_position_description": effective_position_description,
                "rescission_date": rescission_date.strftime("%Y-%m-%d")
                if rescission_date
                else None,
                "notice_date": notice_date.strftime("%Y-%m-%d")
                if notice_date
                else None,
                "target_admission_date": target_admission_date.strftime("%Y-%m-%d")
                if target_admission_date
                else None,
                "code_reason_rescission": code_reason_rescission,
                "activity_name": activity_name,
                "rescission_time": rescission_time,
                "face_to_face_communication": face_to_face_communication,
                "email_corp_witness_1": email_corp_witness_1,
                "email_corp_witness_2": email_corp_witness_2,
                "workplace": workplace,
                "zip_code": zip_code,
                "complete_address": complete_address,
                "private_email": private_email,
                "private_phone": private_phone,
                "corporate_cell_phone": corporate_cell_phone,
                "corporate_email": corporate_email,
                "return_computer": return_computer,
                "return_chair": return_chair,
                "return_other_items": return_other_items,
                "access_data_center": access_data_center,
                "servicenow_sys_id": servicenow_sys_id,
                "servicenow_ticket_number": servicenow_ticket_number,
            },
        )

    def read(self, rescission_uuid: str):
        return self.http.get(
            url=f"{self.severino_api_url}{self.path}/{rescission_uuid}/"
        )

    def list(self, filters: dict = {}):
        """List

        Args:
            filters (dict, optional): List of filters: private_email, private_phone, corporate_cell_phone, corporate_email, servicenow_sys_id, servicenow_ticket_number. E.g: {"private_email": "severino@gmail.com"}
        """
        return self.http.get(url=f"{self.severino_api_url}{self.path}/", params=filters)

    def update(
        self,
        process_code: int = None,
        process_status: int = None,
        start_date: datetime = None,
        completion_date: datetime = None,
        applicant_name: str = None,
        target_company_code: int = None,
        target_enrollment: int = None,
        target_name: str = None,
        effective_position_description: str = None,
        rescission_date: datetime = None,
        notice_date: datetime = None,
        target_admission_date: datetime = None,
        code_reason_rescission: int = None,
        activity_name: str = None,
        rescission_time: str = None,
        face_to_face_communication: str = None,
        email_corp_witness_1: str = None,
        email_corp_witness_2: str = None,
        workplace: str = None,
        zip_code: str = None,
        complete_address: str = None,
        private_email: str = None,
        private_phone: str = None,
        corporate_cell_phone: str = None,
        corporate_email: str = None,
        return_computer: bool = None,
        return_chair: bool = None,
        return_other_items: bool = None,
        access_data_center: bool = None,
        servicenow_sys_id: str = None,
        servicenow_ticket_number: str = None,
    ):
        return self.http.patch(
            url=f"{self.severino_api_url}{self.path}/{process_code}/",
            data={
                "process_status": process_status,
                "start_date": start_date.strftime("%Y-%m-%d") if start_date else None,
                "completion_date": completion_date.strftime("%Y-%m-%d")
                if completion_date
                else None,
                "applicant_name": applicant_name,
                "target_company_code": target_company_code,
                "target_enrollment": target_enrollment,
                "target_name": target_name,
                "effective_position_description": effective_position_description,
                "rescission_date": rescission_date.strftime("%Y-%m-%d")
                if rescission_date
                else None,
                "notice_date": notice_date.strftime("%Y-%m-%d")
                if notice_date
                else None,
                "target_admission_date": target_admission_date.strftime("%Y-%m-%d")
                if target_admission_date
                else None,
                "code_reason_rescission": code_reason_rescission,
                "activity_name": activity_name,
                "rescission_time": rescission_time,
                "face_to_face_communication": face_to_face_communication,
                "email_corp_witness_1": email_corp_witness_1,
                "email_corp_witness_2": email_corp_witness_2,
                "workplace": workplace,
                "zip_code": zip_code,
                "complete_address": complete_address,
                "private_email": private_email,
                "private_phone": private_phone,
                "corporate_cell_phone": corporate_cell_phone,
                "corporate_email": corporate_email,
                "return_computer": return_computer,
                "return_chair": return_chair,
                "return_other_items": return_other_items,
                "access_data_center": access_data_center,
                "servicenow_sys_id": servicenow_sys_id,
                "servicenow_ticket_number": servicenow_ticket_number,
            },
        )

    def partial_update(
        self,
        process_code: int,
        data: dict,
    ):
        return self.http.patch(
            url=f"{self.severino_api_url}{self.path}/{process_code}/", data=data
        )

    def delete(self, rescission_uuid):
        return self.http.delete(
            url=f"{self.severino_api_url}{self.path}/{rescission_uuid}/"
        )

    def change_status(self, process_code: str, sys_status: str):
        status_choices = [
            "waiting",
            "future_dismissal",
            "canceled",
            "not_processed",
            "ready",
            "success",
            "retry",
            "error",
            "error_but_finished",
        ]

        if not sys_status in status_choices:
            raise f"Invalid status, options are: {','.join(status_choices)}"

        data = {"sys_status": sys_status}

        return self.http.patch(
            url=f"{self.severino_api_url}{self.path}/{process_code}/",
            data=data,
        )
